/* $Header:   F:/source/cwright/cwhelp/cwhelp.c_v   1.15   01 Jul 1992 09:23:46   ericj  $ */
/*
 *
 *                  Copyright 1991,1992 Premia Corp.
 *                        All rights reserved
 *
 *    This source code is provided to the purchaser of the product
 *    for the purpose of allowing the purchaser to customize and/or
 *    extend the functionality of the product.  The source code   
 *    remains the property of Premia Corp. with all rights reserved.
 *    Any modifications to the source code are considered derivative
 *    works and all rights thereto are reserved to Premia Corp. except
 *    that the purchaser may use the modified product in the same
 *    permitted manner as the unmodified product.  It is expressly
 *    prohibited to distribute in any manner any part of the original
 *    source code, whether in source or object form, without the
 *    express written permission from Premia Corp.
 *
 *    Owner:
 *        Premia Corporation
 *        1075 NW Murray Blvd., Suite 268
 *        Portland OR 97229
 *
 */


#include <windows.h>
#include <search.h>
#include "exports.h"
#include "startup.h"
#include "cwhelp.h"
#include <string.h>

/*
 * Extended help from customer for MSVC
 */
extern	HelpNameInfo msvc_mfc_calls[];


#define VERSION_30      0x0003

PRIVATE LPSTR _HelpFindKeyname( LPSTR keyname, LPINT infoIndex, LPINT contextId );
PRIVATE int   CDECL XFAR helpKeyCompare( const void XFAR *s1, const void XFAR *s2 );
PRIVATE LPSTR CWRIGHTHelpFileName = "cwhelp.hlp";
PRIVATE void  set_help_info( void );

LPHelpDataInfo help_data_info = NULL;

LPSTR XNEAR win30_sdk_name = "SDKWIN.HLP";
LPSTR XNEAR win31_sdk_name = "WIN31WH.HLP";
LPSTR XNEAR win_help_default_name = NIL;
BOOL  XNEAR useQuickHelp = FALSE;
BOOL  XNEAR useQCWinHelp = FALSE;

/*
 * Extended help from customer for MSVC
 */
LPSTR XNEAR msvc_win_help	= "MSCXX.HLP";
LPSTR XNEAR msvc_afx_help	= "MFC.HLP";

#define HELP_FILE_NAME(i)   ((help_data_info[ i ].helpFileName == (LPSTR)-1) \
								? ((help_data_info == win30_help_data_info)  \
									? win30_sdk_name                         \
									: win31_sdk_name)                        \
								: help_data_info[ i ].helpFileName)

/* NOTE: Filename of (LPSTR)-1 indicates that the default window SDK 
 * help file name should be used.
 */
HelpDataInfo win30_help_data_info[] = {
	/* name array */     /* file name */  /* suffix */       /* Flags */  
	sdk_function,        (LPSTR)-1,        " FUNCTION",       CWHELP_GLOBAL,
	sdk_printer_escape,  (LPSTR)-1,        " PRINTER ESCAPE", CWHELP_GLOBAL,
	sdk_message,         (LPSTR)-1,        " MESSAGE",        CWHELP_GLOBAL,
	sdk_data_structure,  (LPSTR)-1,        " DATA STRUCTURE", CWHELP_GLOBAL,
	sdk_data_type,       (LPSTR)-1,        " DATA TYPE",      CWHELP_GLOBAL,
	sdk_utility_macro,   (LPSTR)-1,        " UTILITY MACRO",  CWHELP_GLOBAL,
	cwright_menu,        "CWHELP.HLP",     NIL,               0,
	cwright_api,         "CWHELP.HLP",     NIL,               0,
	win_winhelp,         "WINHELP.HLP",    NIL,               CWHELP_GLOBAL,
	NIL,                 NIL,              NIL,               0,
};


HelpDataInfo win31_help_data_info[] = {
	/* name array */     /* file name */  /* suffix */       /* Flags */  
	sdk_function,        (LPSTR)-1,        NIL,               CWHELP_GLOBAL,
	sdk_printer_escape,  (LPSTR)-1,        NIL,               CWHELP_GLOBAL,
	sdk_message,         (LPSTR)-1,        NIL,               CWHELP_GLOBAL,
	sdk_data_structure,  (LPSTR)-1,        NIL,               CWHELP_GLOBAL,
	sdk_data_type,       (LPSTR)-1,        NIL,               CWHELP_GLOBAL,
	sdk_utility_macro,   (LPSTR)-1,        NIL,               CWHELP_GLOBAL,
	sdk_win31_calls,     (LPSTR)-1,        NIL,               CWHELP_GLOBAL,
	sdk_win31_functions, (LPSTR)-1,        " FUNCTION",       CWHELP_GLOBAL,
	cwright_menu,        "CWHELP.HLP",     NIL,               0,
	cwright_api,         "CWHELP.HLP",     NIL,               0,
	win_winhelp,         "WINHELP.HLP",    NIL,               CWHELP_GLOBAL,
/*
 * Extended help from customer for MSVC
 */
	msvc_mfc_calls, 	 "MFC.HLP", 	   NIL, 			  0,
	NIL,                 NIL,              NIL,               0,
};

#ifdef __BORLANDC__
LIBMAIN    /* defines the LibMain() function for Borland C */
int DLL WEP(int nParam){return(1);}
#endif


void DLL
_init( void )
{
	LibExport( "CWHelpSDKName LPSTR" );
	LibExport( "int CWHelp LPSTR int" );
	LibExport( "int _HelpExitShutdown" );
	LibExport( "int helpCommand WORD WORD");
	LibExport( "int helpMenuInit WORD WORD WORD");
	LibExport( "CWHelpDefaultName LPSTR" );
	LibExport( "CWUseQuickHelp BOOL" );
	LibExport( "CWUseQCWinHelp BOOL" );

	EventRegister( EVENT_EXIT_SHUTDOWN, EVENT_NORMAL, "_HelpExitShutdown" );
}

PUBLIC void DLL
CWHelpDefaultName( LPSTR name )
{
	if (name && *name)
		win_help_default_name = StrNew( name );
}

PUBLIC void DLL
CWUseQuickHelp( BOOL useqh )
{
	useQuickHelp = useqh;
}

PUBLIC void DLL
CWUseQCWinHelp( BOOL useqh )
{
	useQCWinHelp = useqh;
}


PUBLIC void DLL
CWHelpSDKName( LPSTR name )
{
	set_help_info();
	if (name && *name)
	{
		if (help_data_info == win30_help_data_info)
			win30_sdk_name = StrNew( name );
		else
			win31_sdk_name = StrNew( name );
	}
}


/*
 ** helpCommand
 *
 * Conditionally handle a menu command.  Return non-zero if processed.
 *
 */
PUBLIC int DLL
helpCommand(HWND hWnd, WORD menuID)
{
	switch(menuID)
	{
	case IDM_HELP_COMMANDS:
		CWHelp( (long)"HELP:COMMANDS", HELP_KEY );
		break;
	case IDM_HELP_FUNCTIONS:
		CWHelp( (long)"HELP:FUNCTIONS", HELP_KEY );
		break;
	case IDM_HELP_INDEX:
		CWHelp( (long)"HELP:INDEX", HELP_KEY );
		break;
	case IDM_HELP_ON_HELP:
		WinHelp(WinQHFrame(), NIL, HELP_HELPONHELP, 0L );
		break;
	default:
		return(FALSE);
	}
	return(TRUE);
}

PUBLIC int DLL
helpMenuInit(HWND hWnd, WORD menuID, HMENU hMenu)
{
	if (menuID == HELP_MENU)
	{
		/* menu initialization code goes here */
	}
	return(0);
}


/*
 * Send a HELP_QUIT notification to WinHelp() before exiting to
 * inform it that the help files are no longer required.
 */
int DLL
_HelpExitShutdown( void )
{
	int  i = 0;

	set_help_info();

	while (HELP_FILE_NAME(i))
	{
		WinHelp(WinQHFrame(), HELP_FILE_NAME(i), HELP_QUIT, 0L);
		i++;
	}
	WinHelp(WinQHFrame(), CWRIGHTHelpFileName,HELP_QUIT,0L);
	return 0;
}


/*
 ** CWHelp()
 *
 * PARAMETERS:
 *   key   - mode == HELP_KEY,     points to a keyname to lookup
 *                == HELP_CONTEXT, a 32-bit context-identifier
 *   mode  - 0: 
 *           HELP_KEY:        perform a lookup on the string 'key'
 *           HELP_CONTEXT:    perform a lookup with the context-identifier in key
 *           HELP_HELPONHELP: display help for using help
 *           HELP_INDEX:      display index of CODEWRIGHT help file
 *
 * DESCRIPTION:
 *
 * RETURNS:
 *   TRUE  - if help was called, either with a string or generic call
 *   FALSE - otherwise
 */
BOOL DLL
CWHelp( LONG key, int mode )
{
	LPSTR keyname;
	LPSTR lookupkeyname;
	int   infoIndex;
	BOOL  status    = TRUE;
	int   argc      = ArgCount();
	int   contextId = 0;

	set_help_info();

	if (argc == 1)
	{
		keyname = StrNew( (LPSTR)key );
		goto CWHelp_keylookup;
	}
	else if (!argc)
	{
		keyname = _ReadWord();
CWHelp_keylookup:
		lookupkeyname = _HelpFindKeyname( keyname, &infoIndex, &contextId );
		if (lookupkeyname && *lookupkeyname)
		{
			if (contextId == -1)
				WinHelp(WinQHFrame(), NIL, HELP_HELPONHELP, 0L );
			else if (contextId)
				WinHelp(WinQHFrame(), HELP_FILE_NAME(infoIndex),
					HELP_CONTEXT, contextId );
			else
				{
				char hdirbuf[256] = "";
				if (help_data_info[infoIndex].helpArray == msvc_mfc_calls)	// look for MFC help
					{
					if (GetPrivateProfileString("Microsoft Visual C++", "HelpFile", NULL,
							hdirbuf, 256, "msvc.ini"))
						strcat(hdirbuf, "\\");
					}
				strcat(hdirbuf, HELP_FILE_NAME(infoIndex));
				WinHelp(WinQHFrame(), hdirbuf, HELP_KEY, (DWORD)(LPSTR)lookupkeyname);
				}
			StrFree( lookupkeyname );
		}
		else
		{
			/*
			 *******************************************************
			 * Place any default action here such as calling Quick *
			 * Help with the keyname specified.                    *
			 *******************************************************
			 */
			if (win_help_default_name)
			{
				WinHelp(WinQHFrame(), win_help_default_name, HELP_KEY, (DWORD)(LPSTR)keyname);
			}
			else if (useQuickHelp)
			{
				LPSTR str = StrNew( "qh " );
				str = StrAppend( str, keyname );
				ExecCommand( str );
				StrFree( str );
			}
			/*
			 * Extended help from customer for MSVC
			 */
			else if (useQCWinHelp)
			{
				char hdirbuf[256] = "";
				
				if (GetPrivateProfileString("Microsoft Visual C++", "HelpFile", NULL,
				hdirbuf, 256, "msvc.ini"))
					strcat(hdirbuf, "\\");
				if (StrMatch("(Afx|afx|m_|^C|^~)", keyname, SEARCH_MAXIMAL, NULL))
					strcat(hdirbuf, msvc_afx_help);
				else
					strcat(hdirbuf, msvc_win_help);
				status = WinHelp(WinQHFrame(), hdirbuf, HELP_KEY, (DWORD)(LPSTR)keyname);
			}
			else
				MsgPrintf( WARNING_MSG_LEVEL, "No help available for: %s", keyname );
			status = FALSE;
		}
		StrFree( keyname );
	}
	else
	{
		switch( mode )
		{
			case HELP_KEY:        /* perform a lookup on the string 'key' */
				keyname = StrNew( (LPSTR)key );
				goto CWHelp_keylookup;
			case HELP_CONTEXT:    /* perform a lookup with the context-identifier in key */
				WinHelp(WinQHFrame(), CWRIGHTHelpFileName, mode, key );
				break;
			case HELP_HELPONHELP: /* display help for using help */
				WinHelp(WinQHFrame(), NIL, mode, 0L );
				break;
			case HELP_INDEX:      /* display index of CODEWRIGHT help file */
				WinHelp(WinQHFrame(), CWRIGHTHelpFileName, mode, 0L );
				break;
			default:
				status = FALSE;
				break;
		}
	}
	return status;
}



/*
 ** _HelpFindKeyname()
 *
 * PARAMETERS:
 *   keyname   - name of key to find (i.e. 'GetDC')
 *   infoIndex - pointer to 'int' which will contain the index into
 *               the 'help_data_info' structure where the keyname was found.
 *   contedId  - pointer to 'int' which will contain the help context id.
 *
 * DESCRIPTION:
 *   Scan through each array specified in the help_data_info structure
 *   to find which array contains the specified keyname.  If found,
 *   set *infoIndex to the 'help_data_info' entry and return a pointer
 *   to a string to use in the call to WinHelp().  The string will have
 *   the appropriate suffix appended.
 */
PRIVATE LPSTR
_HelpFindKeyname( LPSTR keyname, LPINT infoIndex, LPINT contextId )
{
	int   dataInfoIndex = 0;
	LPSTR foundKeyname = NIL;
	int   dataItemCount = 0;
	LPSTR found;
	LPHelpNameInfo dataItemArray;

	set_help_info();

	if (!keyname || !*keyname)
		return NIL;

	/* Scan through each array specified in the help_data_info structure
	 * to find which array contains the specified keyname.
	 */
	while (help_data_info[ dataInfoIndex ].helpArray)
	{
		dataItemArray = help_data_info[ dataInfoIndex ].helpArray;
		dataItemCount = 0;

		/* Determine how many entries there are in the current array.
		 */
		while (dataItemArray[ dataItemCount ].helpName)
			dataItemCount++;


		found = (LPSTR)bsearch( keyname, 
					(LPVOID)dataItemArray,
					dataItemCount,
					sizeof( HelpNameInfo ),
					helpKeyCompare );
		if (found)
		{
			/* 'found' points to the data name info that matched.
			 */
			foundKeyname = StrNew( keyname );
			foundKeyname = StrAppend( foundKeyname, 
								help_data_info[ dataInfoIndex ].helpSuffix );
			if (contextId)
				*contextId = ((LPHelpNameInfo)found)->helpContext;

			if (infoIndex)
				*infoIndex = dataInfoIndex;
			break;
		}

		dataInfoIndex++;
	}
	return foundKeyname;
}


/*
 ** helpKeyCompare()
 *
 * PARAMETERS:
 *   s1, s2  - strings to compare
 *
 * DESCRIPTION:
 *   A comparison function passed to bsearch() for comparing
 *   two elements, s1 and s2, in a string array.
 */
PRIVATE int CDECL XFAR 
helpKeyCompare( const void XFAR *s1, const void XFAR *s2 )
{
	LPVOID news2 = (LPVOID*)s2;
	return lstrcmpi( (LPSTR)s1, ((LPHelpNameInfo)news2)->helpName );
}


PRIVATE void
set_help_info( void )
{
	/* Set the help_data_info pointer to the correct help_data_info structure
	 * depending on the version of Window being run.
	 */
	if (GetVersion() == VERSION_30)
		help_data_info = win30_help_data_info;
	else
		help_data_info = win31_help_data_info;
}

